@@ -13,6 +13,7 @@ from scrapy.http import Response
 from bs4 import UnicodeDammit
 from scrapy import Request
 import traceback
+from django.core.serializers.json import DjangoJSONEncoder
 
 import urllib
 import socket
@@ -21,6 +22,10 @@ import json
 import copy
 import pdb
 import redis
+import base64
+
+from functools import partial
+json.dumps = partial(json.dumps, cls=DjangoJSONEncoder)
 
 
 class easyCrawlSpider(RedisCrawlSpider):
@@ -55,6 +60,8 @@ class easyCrawlSpider(RedisCrawlSpider):
     else:
         save_into_priority_queue = (lambda self, server, key_name, val, data: server.zadd(key_name, {data: val}))
 
+    is_base64 = False
+
     def next_requests(self):
         """
         整个的心跳函数
@@ -189,7 +196,10 @@ class easyCrawlSpider(RedisCrawlSpider):
             if req_method.upper() != "GET":
                 post_data = start_map.get("body", "")
                 req_body = post_data
-                req_body = post_data.encode("utf-8")
+                if start_map.get("meta", {}).get("is_base64"):
+                    req_body = base64.b64decode(req_body)
+                else:
+                    req_body = post_data.encode("utf-8")
                 # if isinstance(post_data, dict):
                 #     req_body = urllib.urlencode(post_data)
                 #     # req_body = post_data
@@ -357,7 +367,15 @@ class easyCrawlSpider(RedisCrawlSpider):
         return str/unicode request body -> dict
         """
         r_copy = self.get_common_request(r)
-        return r_copy.body.decode("utf-8")
+        try:
+            return r_copy.body.decode("utf-8")
+        except:
+            # 判断是否二进制数据，如果是二进制数据，需要编码base64
+            if isinstance(r_copy.body, bytes):
+                self.logger.info("Requests body 格式为二进制流，将编码为base64 并解码为utf-8字符串")
+                self.is_base64 = True
+                return base64.b64encode(r_copy.body).decode("utf-8")
+            return r_copy.body
         dict_body = self.parse_query_2_dict(r_copy.body)
 
         # pdb.set_trace()
@@ -437,6 +455,7 @@ class easyCrawlSpider(RedisCrawlSpider):
         # 检测请求体：
         # 如果这个请求不是GET类型，那么需要添加请求体body
         reput_request["body"] = self.get_request_body(r_copy)
+
         if reput_request.get("method") == "GET":
             reput_request.pop("body")
 
@@ -444,6 +463,10 @@ class easyCrawlSpider(RedisCrawlSpider):
         # 更新附加信息
 
         meta = copy.deepcopy(r_copy.meta)
+        # 判断是否是base64二进制编码
+        if self.is_base64:
+            if "is_base64" not in meta:
+                meta["is_base64"] = True
         # pdb.set_trace()
         last_easyspider = meta.get("easyspider") or {}
         last_easyspider.update({
@@ -484,7 +507,7 @@ class easyCrawlSpider(RedisCrawlSpider):
                     try:
                         traceback.print_exc()
                         import pdb
-                        pdb.set_trace()
+                        # pdb.set_trace()
                         exc_info = json.dumps(reput_request["meta"]["easyspider"]["exc_info"], ensure_ascii=False)
                     #     json.dumps(exc_info, ensure_ascii=False)
                     #     json.dumps(reput_request["meta"]["easyspider"], ensure_ascii=False)
@@ -510,6 +533,7 @@ class easyCrawlSpider(RedisCrawlSpider):
                         except:
                             traceback.print_exc()
                             # pdb.set_trace()
+                    self.logger.info("超过重试的最大次数限制，放入到 failed key 中")
                 # elif reput_request.get("priority") != 0:
                 elif reput_request.get("priority") > 0:
                     # 有时候priority还会出现负数的情况
